package cn.kdan.compdf.controller.v1;

import cn.hutool.core.util.ObjectUtil;
import cn.kdan.compdf.base.R;
import cn.kdan.compdf.dto.AddFreePlanDTO;
import cn.kdan.compdf.dto.PlanStatisticsDTO;
import cn.kdan.compdf.exception.CommonException;
import cn.kdan.compdf.service.BackgroundUserBalanceService;
import cn.kdan.compdf.utils.TokenUtil;
import cn.kdan.compdf.vo.BackgroundUserBalanceVO;
import cn.kdan.compdf.vo.BackgroundUserPackageBalanceVO;
import cn.kdan.compdf.vo.BillVO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @author ComPDFKit-WPH 2022-10-14
 */
@RestController
@RequestMapping("/v1/balance")
public class BackgroundUserBalanceController {

    @Value("${background.dashboard.autograph}")
    private String DASHBOARD_SYNC_AUTOGRAPH;

    private final BackgroundUserBalanceService backgroundUserBalanceService;

    public BackgroundUserBalanceController(BackgroundUserBalanceService backgroundUserBalanceService) {
        this.backgroundUserBalanceService = backgroundUserBalanceService;
    }

    /**
     * 获取账户套餐余额
     */
    @GetMapping("/getPackageBalance")
    public R<BackgroundUserPackageBalanceVO> getPackageBalance() {
        BackgroundUserPackageBalanceVO packageBalance = backgroundUserBalanceService.getPackageBalance(TokenUtil.getRequestHeader().getId());
        return R.ok(packageBalance);
    }

    /**
     * 获取账户消费充值记录
     */
    @GetMapping("/getBalanceRecordList")
    public R<IPage<BackgroundUserBalanceVO>> getBalanceRecordList(@RequestParam("page") Integer page, @RequestParam("size") Integer size,Integer timeZone) {
        IPage<BackgroundUserBalanceVO> balanceRecordList = backgroundUserBalanceService.getBalanceRecordList(TokenUtil.getRequestHeader().getId(), page, size,timeZone);
        return R.ok(balanceRecordList);
    }

    /**
     * 账户消费充值记录导出
     */
    @GetMapping("/exportBalanceRecordList")
    public void exportBalanceRecordList(Integer isDecs,Integer timeZone,Boolean isCN,HttpServletResponse response) {
        if (ObjectUtil.isEmpty(isDecs)) {
            isDecs = 1;
        }
        backgroundUserBalanceService.exportBalanceRecordList(timeZone,isDecs,TokenUtil.getRequestHeader().getId(),isCN, response);
    }


    /**
     * plan 消费记录数据同步 一天一次
     */
    @PostMapping("/plansync")
    public R<Void> plansync(@RequestBody List<PlanStatisticsDTO> planStatisticsDTOList, HttpServletRequest request) {
        if (!DASHBOARD_SYNC_AUTOGRAPH.equals(request.getHeader("autograph"))) {
            throw new CommonException("错误");
        }
        backgroundUserBalanceService.planSync(planStatisticsDTOList);
        return R.ok();
    }

    /**
     * FreePlan 免费资产月底跟新记录数据同步
     */
    @PostMapping("/free-plansync")
    public R<Void> freePlanSync(@RequestBody AddFreePlanDTO freePlanDTO, HttpServletRequest request) {
        if (!DASHBOARD_SYNC_AUTOGRAPH.equals(request.getHeader("autograph"))) {
            throw new CommonException("错误");
        }
        backgroundUserBalanceService.freePlanSync(freePlanDTO);
        return R.ok();
    }


    /**
     * 获取套餐
     * @return BillVO
     */
    @GetMapping("/getBill")
    public R<List<BillVO>> getBill() {
        return R.ok(backgroundUserBalanceService.getBill(TokenUtil.getRequestHeader().getId()));
    }

}
