package cn.kdan.compdf.controller.v1;

import cn.kdan.compdf.base.R;
import cn.kdan.compdf.dto.ChangePasswordDTO;
import cn.kdan.compdf.dto.LoginDTO;
import cn.kdan.compdf.dto.LoginUser;
import cn.kdan.compdf.dto.LoginUserDTO;
import cn.kdan.compdf.enums.AuthEnum;
import cn.kdan.compdf.enums.ResponseEnum;
import cn.kdan.compdf.exception.CommonException;
import cn.kdan.compdf.service.BackgroundUserService;
import cn.kdan.compdf.service.UsersService;
import cn.kdan.compdf.utils.TokenUtil;
import cn.kdan.compdf.vo.LoginVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 * @author comPDF-Kit WPH 2022-10-09
 */
@Slf4j
@RestController
@RequestMapping("/v1/user")
public class BackgroundUserController {

    private final UsersService usersService;

    private final BackgroundUserService backgroundUserService;

    public BackgroundUserController(UsersService usersService, BackgroundUserService backgroundUserService) {
        this.usersService = usersService;
        this.backgroundUserService = backgroundUserService;
    }

    /**
     * 用户登录
     */
    @PostMapping("/login")
    public R<LoginVO> login(@Validated @RequestBody LoginDTO loginDTO) {
        return backgroundUserService.login(loginDTO);
    }

    @PostMapping("/logout")
    public R<Void> logout(HttpServletRequest request) {
        String token = request.getHeader(AuthEnum.AUTHORIZATION);
        log.info("登出token：{}，time：{}", token, new Date());
        backgroundUserService.logout(token.replaceFirst("Bearer ",""));
        return R.ok();
    }

    @PutMapping("/changePassword")
    public R<Void> changePassword(@Validated @RequestBody ChangePasswordDTO changePasswordDTO, HttpServletRequest request) {
        String token = request.getHeader(AuthEnum.AUTHORIZATION);
        backgroundUserService.changePassword(changePasswordDTO, token.replaceFirst("Bearer ",""));
        return R.ok();
    }

    /**
     * userFirstLogin
     */
    @GetMapping("/userFirstLogin")
    public R<String> userFirstLogin(){
        return R.ok(backgroundUserService.getUSerFirstLogin(TokenUtil.getRequestHeader().getTenantId()).toString());
    }

    /**
     * 通过邮箱（用户名）获取用户信息
     */
    @PostMapping("/getUserInfo")
    public R<LoginUserDTO> getUserInfo(@RequestBody LoginUser loginUser){
        return R.ok(backgroundUserService.getUserInfo(loginUser));
    }

    /**
     * 通过邮箱（用户名）创建用户
     */
    @PostMapping("/creatUser")
    R<LoginUserDTO> creatUser(@RequestBody LoginUser loginUser){
        return R.ok(backgroundUserService.creatUser(loginUser));
    }

    @PostMapping("/updateName")
    public R<Void> updateFullName(@RequestParam String fullName) {
        if (StringUtils.isEmpty(fullName)){
            throw new CommonException(ResponseEnum.ERROR_NEW_NAME_CANNOT_BE_EMPTY);
        }
        usersService.updateFullName(TokenUtil.getRequestHeader().getUsersId(),fullName);
        return R.ok();
    }

    @GetMapping("/verify")
    public R<Void> verify(){
        return R.ok();
    }

}
