package cn.kdan.compdf.controller.v1;

import cn.hutool.core.util.ObjectUtil;
import cn.kdan.compdf.base.R;
import cn.kdan.compdf.dto.AddNewWebhookDTO;
import cn.kdan.compdf.dto.EditWebhookDTO;
import cn.kdan.compdf.dto.WebhookLogsDTO;
import cn.kdan.compdf.service.BackgroundUserWebhookService;
import cn.kdan.compdf.utils.TokenUtil;
import cn.kdan.compdf.vo.BackgroundEventVO;
import cn.kdan.compdf.vo.BackgroundUserWebhookInfoVO;
import cn.kdan.compdf.vo.BackgroundUserWebhookVO;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * webhooks接口
 * @author comPDF-Kit WPH 2022-10-09
 */
@RestController
@RequestMapping("/v1/webhook")
public class BackgroundUserWebhookController {

    private final BackgroundUserWebhookService backgroundUserWebhookService;

    public BackgroundUserWebhookController(BackgroundUserWebhookService backgroundUserWebhookService) {
        this.backgroundUserWebhookService = backgroundUserWebhookService;
    }

    /**
     * 获取Webhooks列表
     */
    @GetMapping("/getWebhookList")
    public R<List<BackgroundUserWebhookVO>> getWebhookList(Integer timeZone) {
        if (ObjectUtil.isEmpty(timeZone)) {
            timeZone = 0;
        }
        return R.ok(backgroundUserWebhookService.getWebhookList(TokenUtil.getRequestHeader().getId(),timeZone));
    }

    /**
     * 获取系统事件，新增编辑Webhook页面调用
     */
    @GetMapping("/getEventList")
    public R<List<BackgroundEventVO>> getEventList() {
        return R.ok(backgroundUserWebhookService.getEventList());
    }

    /**
     * 编辑Webhook时，查询已勾选事件id
     */
    @GetMapping("/getEnableEventIdList/{webhookId}")
    public R<List<Long>> getEnableEventIdList(@PathVariable("webhookId") Long webhookId) {
        return R.ok(backgroundUserWebhookService.getEnableEventIdList(webhookId));
    }

    /**
     * 获取Webhook信息
     */
    @GetMapping("/getWebhookInfoById/{webhookId}")
    public R<BackgroundUserWebhookInfoVO> getWebhookInfoById(@PathVariable("webhookId") Long webhookId) {
        return R.ok(backgroundUserWebhookService.getWebhookInfoById(webhookId));
    }

    /**
     * 添加新Webhook
     */
    @PostMapping("/addNewWebhook")
    public R<Void> addNewWebhook(@Validated @RequestBody AddNewWebhookDTO addNewWebhookDTO) {
        addNewWebhookDTO.setUserId(TokenUtil.getRequestHeader().getId());
        addNewWebhookDTO.setTenantId(TokenUtil.getRequestHeader().getTenantId());
        backgroundUserWebhookService.addNewWebhook(addNewWebhookDTO);
        return R.ok();
    }

    /**
     * 修改
     */
    @PutMapping("/editWebhook")
    public R<Void> editWebhook(@Validated @RequestBody EditWebhookDTO editWebhookDTO) {
        editWebhookDTO.setUserId(TokenUtil.getRequestHeader().getId());
        backgroundUserWebhookService.editWebhook(editWebhookDTO);
        return R.ok();
    }

    /**
     * 删除
     */
    @DeleteMapping("deleteById/{id}")
    public R<Void> deleteById(@PathVariable("id") Long id) {
        backgroundUserWebhookService.deleteById(id);
        return R.ok();
    }

    @PostMapping("/last-send-sync")
    public R<Void> updateWebhookLastSendTime(@RequestBody List<WebhookLogsDTO> webhookLogsDTOList){
        backgroundUserWebhookService.updateWebhookLastSendTime(webhookLogsDTOList);
        return R.ok();
    }

    /**
     * editWebhookStatus 修改webhook状态
     * @param id webhookId
     * @param status 状态， -1：关闭，0：删除，1：正常
     * @return Void
     */
    @PutMapping("/editWebhookStatus")
    public R<Void> editWebhookStatus(Long id,Integer status){
        backgroundUserWebhookService.updateWebhookStatus(id,status);
        return R.ok();
    }


}
