package cn.kdan.compdf.controller.v1;

import cn.kdan.compdf.base.R;
import cn.kdan.compdf.dto.ApplyInvoiceDTO;
import cn.kdan.compdf.dto.BillSendEmailDTO;
import cn.kdan.compdf.dto.UserBillingUpInfoDTO;
import cn.kdan.compdf.enums.ResponseEnum;
import cn.kdan.compdf.exception.CommonException;
import cn.kdan.compdf.service.OrderCashFlowService;
import cn.kdan.compdf.service.UserBillingInformationService;
import cn.kdan.compdf.utils.TokenUtil;
import cn.kdan.compdf.vo.ApplyInvoiceVO;
import cn.kdan.compdf.vo.OrderBillVo;
import cn.kdan.compdf.vo.UserBillingInfoVO;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author ComPDFKit-WPH 2023/5/15
 * <p>
 * 用户账单接口
 */
@RestController
@RequestMapping("/v1/user/billing")
@RequiredArgsConstructor
public class UserBillingController {

    private final UserBillingInformationService billingInformationService;

    /**
     * 获取用户账单信息
     *
     * @return UserBillingInfoVO
     */
    @GetMapping("/info")
    public R<UserBillingInfoVO> getUserBillingInfo() {
        return R.ok(billingInformationService.getUserBillingInfo(TokenUtil.getRequestHeader().getUsersId()));
    }

    /**
     * 获取用户订单流水
     *
     * @return OrderBillVo
     */
    @GetMapping("/getOrderBillVos")
    public R<List<OrderBillVo>> getOrderBillVos() {
        return R.ok(billingInformationService.getOrderBillVos(TokenUtil.getRequestHeader().getUsersId()));
    }


    /**
     * 更新用户账单信息
     *
     * @param userBillingUpInfoDTO userBillingUpInfoDTO
     */
    @PostMapping("/update")
    public R<Void> updateUserBillingInfo(@RequestBody UserBillingUpInfoDTO userBillingUpInfoDTO) {
        billingInformationService.updateUserBillingInfo(TokenUtil.getRequestHeader().getUsersId(), userBillingUpInfoDTO);
        return R.ok();
    }

    /**
     * 修改用户账单发送email
     *
     * @param email email
     */
    @PostMapping("/update/email")
    public R<Void> updateUserBillingEmail(String email) {
        if (StringUtils.isEmpty(email)) {
            throw new CommonException(ResponseEnum.ERROR_BILL_EMAIL_NULL);
        }
        billingInformationService.updateUserBillingEmail(TokenUtil.getRequestHeader().getUsersId(), email);
        return R.ok();
    }

    /**
     * 申请开发票
     *
     * @param applyInvoiceDTO applyInvoiceDTO
     * @return ApplyInvoiceVO
     */
    @PostMapping("/applyInvoice")
    public R<ApplyInvoiceVO> applyInvoice(@RequestBody ApplyInvoiceDTO applyInvoiceDTO) {
        return R.ok(billingInformationService.applyInvoice(TokenUtil.getRequestHeader().getUsersId(), applyInvoiceDTO));
    }

    /**
     * bill发票邮件发送
     *
     * @param sendEmailDTO billSendEmailDTO
     */
    @PostMapping("/billSendEmail")
    public R<Void> billSendEmail(@RequestBody BillSendEmailDTO sendEmailDTO) {
        billingInformationService.billSendEmail(sendEmailDTO);
        return R.ok();
    }


}
