package cn.kdan.compdf.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.kdan.compdf.config.ExcelWidthStyleStrategy;
import cn.kdan.compdf.config.RedisConstantServer;
import cn.kdan.compdf.dto.DashExportXlsxDTO;
import cn.kdan.compdf.dto.DashboardStatisticsDTO;
import cn.kdan.compdf.entity.BackgroundConvertData;
import cn.kdan.compdf.entity.dto.AnalysisDataDTO;
import cn.kdan.compdf.entity.dto.DashboardDetailedDataDTO;
import cn.kdan.compdf.entity.dto.DashboardExportObject;
import cn.kdan.compdf.entity.dto.DashboardQueryDTO;
import cn.kdan.compdf.enums.BackgroundErrorEnum;
import cn.kdan.compdf.enums.ResponseEnum;
import cn.kdan.compdf.exception.CommonException;
import cn.kdan.compdf.mapper.BackgroundConvertDataMapper;
import cn.kdan.compdf.service.BackgroundConvertDataService;
import cn.kdan.compdf.service.BackgroundToolsService;
import cn.kdan.compdf.service.BackgroundUserProjectService;
import cn.kdan.compdf.service.BackgroundUserService;
import cn.kdan.compdf.utils.MyLocalDateUtil;
import cn.kdan.compdf.utils.TokenUtil;
import cn.kdan.compdf.vo.ProjectQueryVO;
import cn.kdan.compdf.vo.ToolQueryVO;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.util.MapUtils;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.WriteTable;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.cache.CacheManager;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author comPDF-Kit WPH 2022-10-09
 */
@Service
@Slf4j
public class BackgroundConvertDataServiceImpl extends ServiceImpl<BackgroundConvertDataMapper, BackgroundConvertData> implements BackgroundConvertDataService {

    @Resource(name = "myCacheManager")
    private CacheManager cacheManager;

    private final StringRedisTemplate redisTemplate;

    private final BackgroundUserService userService;

    private final BackgroundUserProjectService backgroundUserProjectService;
    private final BackgroundToolsService toolsService;

    private static final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy/MM/dd");

    public BackgroundConvertDataServiceImpl(StringRedisTemplate redisTemplate, BackgroundUserService userService, BackgroundUserProjectService backgroundUserProjectService, BackgroundToolsService toolsService) {
        this.redisTemplate = redisTemplate;
        this.userService = userService;
        this.backgroundUserProjectService = backgroundUserProjectService;
        this.toolsService = toolsService;
    }

    @Override
    public IPage<BackgroundConvertData> page(BackgroundConvertData query) {
        return this.page(new Page<>(query.getPage(), query.getSize()), Wrappers.query(query));
    }

    @Override
//    @Cacheable(value = RedisConstantServer.DASHBOARD_ANALYSIS_DATA, cacheManager = "myCacheManager", key = "T(String).valueOf(#queryDTO.toString())", unless = "#result == null")
    public AnalysisDataDTO getAnalysisData(DashboardQueryDTO queryDTO) {
        // 获取一个查询query
        LambdaQueryWrapper<BackgroundConvertData> queryWrapper = getDashboardQueryWrapper(queryDTO);
        long fileTotal = 0; // 文件总数
        long successfulRequest = 0; // 成功数
        long invalidRequest = 0; // 无效数
        long errorRequest = 0;  // 失败数
        BigDecimal errorRatio = BigDecimal.ZERO;  // 失败率
        BigDecimal averageProcessTime = BigDecimal.ZERO;  // 平均处理时长

        List<BackgroundConvertData> backgroundConvertData = this.baseMapper.selectList(queryWrapper);
        if (backgroundConvertData == null || backgroundConvertData.size() <= 0) {
            return AnalysisDataDTO.builder()
                    .fileTotal(fileTotal)
                    .successfulRequest(successfulRequest)
                    .invalidRequest(invalidRequest)
                    .errorRequest(errorRequest)
                    .errorRatio(errorRatio + "%")
                    .averageProcessTime(averageProcessTime + "ms")
                    .build();
        }

        BigDecimal averageProcessTimeTotal = BigDecimal.ZERO; // 总处理时长
        for (BackgroundConvertData backgroundConvertDatum : backgroundConvertData) {
            fileTotal += backgroundConvertDatum.getFileTotal();
            successfulRequest += backgroundConvertDatum.getFileSuccessTotal();
            invalidRequest += backgroundConvertDatum.getFileInvalidTotal();
            errorRequest += backgroundConvertDatum.getFileFailedTotal();
            averageProcessTimeTotal = backgroundConvertDatum.getFileConvertTime().add(averageProcessTimeTotal);
        }
        BigDecimal averageProcessTime1 = averageProcessTimeTotal.divide(new BigDecimal(backgroundConvertData.size()), 2, BigDecimal.ROUND_HALF_UP);
        return AnalysisDataDTO.builder()
                .fileTotal(fileTotal)
                .successfulRequest(successfulRequest)
                .invalidRequest(invalidRequest)
                .errorRequest(errorRequest)
                .errorRatio(new BigDecimal(String.format("%.2f", errorRequest * 100.00 / fileTotal)) + "%")
                .averageProcessTime(TimeConvert(averageProcessTime1, null))
                .build();
    }

    /**
     * ms-s-min-时间处理
     * @param time 毫秒时间
     * @param type 指定转换类型1：s；2：min
     */
    private String TimeConvert(BigDecimal time, Integer type) {
        BigDecimal convertTime = time;
        String convertTimeString = time + "ms";
        // 自动转换
        if (null == type) {
            // ms-s，大于1000毫秒转换
            if (convertTime.compareTo(new BigDecimal("1000")) >= 0) {
                convertTime = convertTime.divide(new BigDecimal("1000"), 2, BigDecimal.ROUND_HALF_UP);
                convertTimeString = convertTime + "s";
                // s-min，大于60秒转换
                if (convertTime.compareTo(new BigDecimal("60")) >= 0) {
                    convertTime = convertTime.divide(new BigDecimal("60"), 2, BigDecimal.ROUND_HALF_UP);
                    convertTimeString = convertTime + "min";
                }
            }
        } else if (type == 1) {
            // 转换为s
            convertTimeString = convertTime.divide(new BigDecimal("1000"), 2, BigDecimal.ROUND_HALF_UP) + "s";
        } else if (type == 2) {
            // 转换为min
            convertTime = convertTime.divide(new BigDecimal("1000"), 2, BigDecimal.ROUND_HALF_UP);
            convertTimeString = convertTime.divide(new BigDecimal("60"), 2, BigDecimal.ROUND_HALF_UP) + "min";
        }
        return convertTimeString;
    }

    @Override
//    @Cacheable(value = RedisConstantServer.DASHBOARD_SUCCESS_DATA, cacheManager = "myCacheManager", key = "T(String).valueOf(#queryDTO.toString())", unless = "#result == null")
    public List<DashboardDetailedDataDTO> getSuccessfulRequestDetailedData(DashboardQueryDTO queryDTO) {
        LambdaQueryWrapper<BackgroundConvertData> queryWrapper = getDashboardQueryWrapper(queryDTO);
        // 添加select 查询参数
        queryWrapper.select(BackgroundConvertData::getFileSuccessTotal, BackgroundConvertData::getSyncTime);
        // 查询数据
        List<BackgroundConvertData> backgroundConvertData = this.baseMapper.selectList(queryWrapper);

        LocalDateTime nowTime = MyLocalDateUtil.getLocalDataTime();
        // 返回数据集合
        List<DashboardDetailedDataDTO> dataDTOS = new ArrayList<>();

        long successTotal = 0;
        // 判断查询类型
        switch (queryDTO.getQueryType()) {
            case 0:
                if (queryDTO.getDays() == 1) {
                    for (int i = 0; i < 24; i++) {
                        // 减去 i 小时时间
                        LocalDateTime minus = nowTime.minus(i, ChronoUnit.HOURS);
                        successTotal = backgroundConvertData.stream().filter(convertData -> MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), minus)
                                && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), minus.plusHours(1)))
                                .collect(Collectors.summarizingLong(BackgroundConvertData::getFileSuccessTotal)).getSum();

                        dataDTOS.add(DashboardDetailedDataDTO.builder()
                                .xData(MyLocalDateUtil.timeZoneConversion(minus, queryDTO.getTimeZone()).toString())
                                .yData(successTotal + "").build());
                    }
                } else {
                    for (int i = 0; i < queryDTO.getDays(); i++) {
                        // 减去 i 天时间
                        LocalDate minus = nowTime.minus(i, ChronoUnit.DAYS).toLocalDate();
                        successTotal = backgroundConvertData.stream()
                                .filter(convertData ->
                                        MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus,-queryDTO.getTimeZone())) &&
                                       !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus.plusDays(1),-queryDTO.getTimeZone())))
                                .collect(Collectors.summarizingLong(BackgroundConvertData::getFileSuccessTotal)).getSum();

                        dataDTOS.add(DashboardDetailedDataDTO.builder()
                                .xData(minus.toString())
                                .yData(successTotal + "").build());
                    }
                }
                break;
            case 1:
                LocalDate startDateTime = queryDTO.getStartDateTime();
                LocalDate endDateTime = queryDTO.getEndDateTime();
                long days = endDateTime.toEpochDay() - startDateTime.toEpochDay()  +1 ; //相差的天数
                if (days == 1L) {
                    endDateTime = endDateTime.plusDays(1L);
                    for (int i = 1; i <= 24; i++) {
                        // 减去 i 小时时间
                        LocalDateTime minus = endDateTime.atStartOfDay().minus(i, ChronoUnit.HOURS);
                        successTotal = backgroundConvertData.stream().filter(convertData ->
                                MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus,-queryDTO.getTimeZone()))
                                && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus.plus(1L, ChronoUnit.HOURS),-queryDTO.getTimeZone())))
                                .collect(Collectors.summarizingLong(BackgroundConvertData::getFileSuccessTotal)).getSum();

                        dataDTOS.add(DashboardDetailedDataDTO.builder()
                                .xData(minus.toString())
                                .yData(successTotal + "").build());
                    }
                } else {
                    for (int i = 0; i < days; i++) {
                        // 减去 i 天时间
                        LocalDate minus = endDateTime.minus(i, ChronoUnit.DAYS);
                        successTotal = backgroundConvertData.stream().filter(convertData ->
                                MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus,-queryDTO.getTimeZone()))
                                && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus.plusDays(1),-queryDTO.getTimeZone())))
                                .collect(Collectors.summarizingLong(BackgroundConvertData::getFileSuccessTotal)).getSum();
                        dataDTOS.add(DashboardDetailedDataDTO.builder()
                                .xData(minus.toString())
                                .yData(successTotal + "").build());
                    }
                }
                break;
            default:
                throw new CommonException(ResponseEnum.SYSTEM_ERROR);
        }
        Collections.reverse(dataDTOS);
        return dataDTOS;
    }

    @Override
    public List<DashboardDetailedDataDTO> getInvalidRequestDetailedData(DashboardQueryDTO queryDTO) {
        LambdaQueryWrapper<BackgroundConvertData> queryWrapper = getDashboardQueryWrapper(queryDTO);
        // 添加select 查询参数
        queryWrapper.select(BackgroundConvertData::getFileInvalidTotal, BackgroundConvertData::getSyncTime);
        // 查询数据
        List<BackgroundConvertData> backgroundConvertData = this.baseMapper.selectList(queryWrapper);

        LocalDateTime nowTime = MyLocalDateUtil.getLocalDataTime();
        // 返回数据集合
        List<DashboardDetailedDataDTO> dataDTOS = new ArrayList<>();

        long successTotal = 0;
        // 判断查询类型
        switch (queryDTO.getQueryType()) {
            case 0:
                if (queryDTO.getDays() == 1) {
                    for (int i = 0; i < 24; i++) {
                        // 减去 i 小时时间
                        LocalDateTime minus = nowTime.minus(i, ChronoUnit.HOURS);
                        successTotal = backgroundConvertData.stream().filter(convertData -> MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), minus)
                                        && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), minus.plusHours(1)))
                                .collect(Collectors.summarizingLong(BackgroundConvertData::getFileInvalidTotal)).getSum();

                        dataDTOS.add(DashboardDetailedDataDTO.builder()
                                .xData(MyLocalDateUtil.timeZoneConversion(minus, queryDTO.getTimeZone()).toString())
                                .yData(successTotal + "").build());
                    }
                } else {
                    for (int i = 0; i < queryDTO.getDays(); i++) {
                        // 减去 i 天时间
                        LocalDate minus = nowTime.minus(i, ChronoUnit.DAYS).toLocalDate();
                        successTotal = backgroundConvertData.stream()
                                .filter(convertData ->
                                        MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus,-queryDTO.getTimeZone())) &&
                                                !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus.plusDays(1),-queryDTO.getTimeZone())))
                                .collect(Collectors.summarizingLong(BackgroundConvertData::getFileInvalidTotal)).getSum();

                        dataDTOS.add(DashboardDetailedDataDTO.builder()
                                .xData(minus.toString())
                                .yData(successTotal + "").build());
                    }
                }
                break;
            case 1:
                LocalDate startDateTime = queryDTO.getStartDateTime();
                LocalDate endDateTime = queryDTO.getEndDateTime();
                long days = endDateTime.toEpochDay() - startDateTime.toEpochDay()  +1 ; //相差的天数
                if (days == 1L) {
                    endDateTime = endDateTime.plusDays(1L);
                    for (int i = 1; i <= 24; i++) {
                        // 减去 i 小时时间
                        LocalDateTime minus = endDateTime.atStartOfDay().minus(i, ChronoUnit.HOURS);
                        successTotal = backgroundConvertData.stream().filter(convertData ->
                                        MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus,-queryDTO.getTimeZone()))
                                                && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus.plus(1L, ChronoUnit.HOURS),-queryDTO.getTimeZone())))
                                .collect(Collectors.summarizingLong(BackgroundConvertData::getFileInvalidTotal)).getSum();

                        dataDTOS.add(DashboardDetailedDataDTO.builder()
                                .xData(minus.toString())
                                .yData(successTotal + "").build());
                    }
                } else {
                    for (int i = 0; i < days; i++) {
                        // 减去 i 天时间
                        LocalDate minus = endDateTime.minus(i, ChronoUnit.DAYS);
                        successTotal = backgroundConvertData.stream().filter(convertData ->
                                        MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus,-queryDTO.getTimeZone()))
                                                && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus.plusDays(1),-queryDTO.getTimeZone())))
                                .collect(Collectors.summarizingLong(BackgroundConvertData::getFileInvalidTotal)).getSum();
                        dataDTOS.add(DashboardDetailedDataDTO.builder()
                                .xData(minus.toString())
                                .yData(successTotal + "").build());
                    }
                }
                break;
            default:
                throw new CommonException(ResponseEnum.SYSTEM_ERROR);
        }
        Collections.reverse(dataDTOS);
        return dataDTOS;
    }

    @Override
//    @Cacheable(value = RedisConstantServer.DASHBOARD_FAIL_DATA, cacheManager = "myCacheManager", key = "T(String).valueOf(#queryDTO.toString())", unless = "#result == null")
    public List<DashboardDetailedDataDTO> getErrorRequestDetailedData(DashboardQueryDTO queryDTO) {
        LambdaQueryWrapper<BackgroundConvertData> queryWrapper = getDashboardQueryWrapper(queryDTO);
        // 添加select 查询参数
        queryWrapper.select(BackgroundConvertData::getFileFailedTotal, BackgroundConvertData::getSyncTime);
        // 查询数据
        List<BackgroundConvertData> backgroundConvertData = this.baseMapper.selectList(queryWrapper);
        // 判断查询类型
        LocalDateTime nowTime = MyLocalDateUtil.getLocalDataTime();
        // 返回数据集合
        List<DashboardDetailedDataDTO> dataDTOS = new ArrayList<>();

        long fileFailedTotal = 0;
        switch (queryDTO.getQueryType()) {
            case 0:
                if (queryDTO.getDays() == 1) {
                    for (int i = 0; i < 24; i++) {
                        // 减去 i 小时时间
                        LocalDateTime minus = nowTime.minus(i, ChronoUnit.HOURS);

                        fileFailedTotal = backgroundConvertData.stream().filter(convertData ->
                                MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), minus)
                                && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), minus.plusHours(1)))
                                .collect(Collectors.summarizingLong(BackgroundConvertData::getFileFailedTotal)).getSum();

                        dataDTOS.add(DashboardDetailedDataDTO.builder()
                                .xData(MyLocalDateUtil.timeZoneConversion(minus, queryDTO.getTimeZone()).toString())
                                .yData(fileFailedTotal + "").build());
                    }
                } else {
                    for (int i = 0; i < queryDTO.getDays(); i++) {
                        // 减去 i 天时间
                        LocalDate minus = nowTime.minus(i, ChronoUnit.DAYS).toLocalDate();

                        fileFailedTotal = backgroundConvertData.stream().filter(convertData ->
                                MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus,-queryDTO.getTimeZone()))
                                && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus.plusDays(1),-queryDTO.getTimeZone())))
                                .collect(Collectors.summarizingLong(BackgroundConvertData::getFileFailedTotal)).getSum();

                        dataDTOS.add(DashboardDetailedDataDTO.builder()
                                .xData(minus.toString())
                                .yData(fileFailedTotal + "").build());
                    }
                }
                break;
            case 1:
                LocalDate startDateTime = queryDTO.getStartDateTime();
                LocalDate endDateTime = queryDTO.getEndDateTime();
                long days = endDateTime.toEpochDay() - startDateTime.toEpochDay()  +1 ; //相差的天数
                if (days == 1L) {
                    endDateTime = endDateTime.plusDays(1L);
                    for (int i = 1; i <= 24; i++) {
                        // 减去 i 小时时间
                        LocalDateTime minus = endDateTime.atStartOfDay().minus(i, ChronoUnit.HOURS);
                        fileFailedTotal = backgroundConvertData.stream().filter(convertData ->
                                MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus,-queryDTO.getTimeZone()))
                                && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus.plus(1L, ChronoUnit.HOURS),-queryDTO.getTimeZone())))
                                .collect(Collectors.summarizingLong(BackgroundConvertData::getFileFailedTotal)).getSum();
                        dataDTOS.add(DashboardDetailedDataDTO.builder()
                                .xData(minus.toString())
                                .yData(fileFailedTotal + "").build());
                    }
                } else {
                    for (int i = 0; i < days; i++) {
                        // 减去 i 天时间
                        LocalDate minus = endDateTime.minus(i, ChronoUnit.DAYS);
                        fileFailedTotal = backgroundConvertData.stream().filter(convertData ->
                                MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus,-queryDTO.getTimeZone()))
                                && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus.plusDays(1),-queryDTO.getTimeZone())))
                                .collect(Collectors.summarizingLong(BackgroundConvertData::getFileFailedTotal)).getSum();
                        dataDTOS.add(DashboardDetailedDataDTO.builder()
                                .xData(minus.toString())
                                .yData(fileFailedTotal + "").build());
                    }
                }
                break;
            default:
                throw new CommonException(ResponseEnum.SYSTEM_ERROR);
        }
        Collections.reverse(dataDTOS);
        return dataDTOS;
    }

    @Override
//    @Cacheable(value = RedisConstantServer.DASHBOARD_ERROR_RATIO_DATA, cacheManager = "myCacheManager", key = "T(String).valueOf(#queryDTO.toString())", unless = "#result == null")
    public List<DashboardDetailedDataDTO> getErrorRatioDetailedData(DashboardQueryDTO queryDTO) {
        List<DashboardDetailedDataDTO> successfulRequestDetailedData = this.getSuccessfulRequestDetailedData(queryDTO);
        List<DashboardDetailedDataDTO> errorRequestDetailedData = this.getErrorRequestDetailedData(queryDTO);
        List<DashboardDetailedDataDTO> dataDTOS = new ArrayList<>();
        for (int i = 0; i < successfulRequestDetailedData.size(); i++) {
            int fileTotal = Integer.parseInt(successfulRequestDetailedData.get(i).getYData()) + Integer.parseInt(errorRequestDetailedData.get(i).getYData());
            dataDTOS.add(DashboardDetailedDataDTO.builder()
                    .xData(successfulRequestDetailedData.get(i).getXData())
                    .yData(fileTotal == 0 ? "0.00%" : new BigDecimal(String.format("%.2f", Integer.parseInt(errorRequestDetailedData.get(i).getYData()) * 100.00
                            / fileTotal)).toString() + "%")
                    .build());
        }
        return dataDTOS;
    }

    @Override
//    @Cacheable(value = RedisConstantServer.DASHBOARD_AVERAGE_PROCESS_DATA, cacheManager = "myCacheManager", key = "T(String).valueOf(#queryDTO.toString())", unless = "#result == null")
    public List<DashboardDetailedDataDTO> getAverageProcessTimeDetailedData(DashboardQueryDTO queryDTO) {
        LambdaQueryWrapper<BackgroundConvertData> queryWrapper = getDashboardQueryWrapper(queryDTO);
        // 添加select 查询参数
        queryWrapper.select(BackgroundConvertData::getFileConvertTime, BackgroundConvertData::getSyncTime);
        // 查询数据
        List<BackgroundConvertData> backgroundConvertData = this.baseMapper.selectList(queryWrapper);
        // 判断查询类型
        LocalDateTime nowTime = MyLocalDateUtil.getLocalDataTime();;
        // 返回数据集合
        List<DashboardDetailedDataDTO> dataDTOS = new ArrayList<>();

        BigDecimal convertTime = BigDecimal.ZERO;
        switch (queryDTO.getQueryType()) {
            case 0:
                if (queryDTO.getDays() == 1) {
                    for (int i = 0; i < 24; i++) {
                        // 减去 i 小时时间
                        LocalDateTime minus = nowTime.minus(i, ChronoUnit.HOURS);
                        long count = backgroundConvertData.stream().filter(convertData ->
                                MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), minus)
                                && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), minus.plusHours(1)) && convertData.getFileConvertTime() != null &&
                                convertData.getFileConvertTime().compareTo(BigDecimal.ZERO) > 0).count();
                        if (count <= 0) {
                            convertTime = BigDecimal.ZERO;
                        } else {
                            convertTime = backgroundConvertData.stream().filter(convertData -> MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), minus)
                                    && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), minus.plusHours(1)) && convertData.getFileConvertTime() != null &&
                                    convertData.getFileConvertTime().compareTo(BigDecimal.ZERO) > 0)
                                    .map(BackgroundConvertData::getFileConvertTime).reduce(BigDecimal.ZERO, BigDecimal::add)
                                    .divide(BigDecimal.valueOf(count), 2, BigDecimal.ROUND_HALF_UP);
                        }
                        dataDTOS.add(DashboardDetailedDataDTO.builder()
                                .xData(MyLocalDateUtil.timeZoneConversion(minus, queryDTO.getTimeZone()).toString())
                                .yData(convertTime.stripTrailingZeros().toPlainString()).build());
                    }
                } else {
                    for (int i = 0; i < queryDTO.getDays(); i++) {
                        // 减去 i 天时间
                        LocalDate minus = nowTime.minus(i, ChronoUnit.DAYS).toLocalDate();
                        long count = backgroundConvertData.stream().filter(convertData ->
                                MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus,-queryDTO.getTimeZone()))
                                && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(),MyLocalDateUtil.timeZoneConversion(minus.plusDays(1),-queryDTO.getTimeZone()))
                                        && convertData.getFileConvertTime() != null &&
                                convertData.getFileConvertTime().compareTo(BigDecimal.ZERO) > 0).count();
                        if (count <= 0) {
                            convertTime = BigDecimal.ZERO;
                        } else {
                            convertTime = backgroundConvertData.stream().filter(convertData ->
                                    MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus,-queryDTO.getTimeZone()))
                                            && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(),MyLocalDateUtil.timeZoneConversion(minus.plusDays(1),-queryDTO.getTimeZone()))
                                            && convertData.getFileConvertTime() != null &&
                                    convertData.getFileConvertTime().compareTo(BigDecimal.ZERO) > 0)
                                    .map(BackgroundConvertData::getFileConvertTime).reduce(BigDecimal.ZERO, BigDecimal::add)
                                    .divide(BigDecimal.valueOf(count), 2, BigDecimal.ROUND_HALF_UP);
                        }
                        dataDTOS.add(DashboardDetailedDataDTO.builder()
                                .xData(minus.toString())
                                .yData(convertTime.stripTrailingZeros().toPlainString()).build());
                    }
                }
                break;
            case 1:
                LocalDate startDateTime = queryDTO.getStartDateTime();
                LocalDate endDateTime = queryDTO.getEndDateTime();
                long days = endDateTime.toEpochDay() - startDateTime.toEpochDay()  +1 ; //相差的天数
                if (days == 1L) {
                    endDateTime = endDateTime.plusDays(1L);
                    for (int i = 1; i <= 24; i++) {
                        // 减去 i 小时时间
                        LocalDateTime minus = endDateTime.atStartOfDay().minus(i, ChronoUnit.HOURS);
                        long count = backgroundConvertData.stream().filter(convertData ->
                                MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus,-queryDTO.getTimeZone()))
                                && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus.plus(1L, ChronoUnit.HOURS),-queryDTO.getTimeZone()))
                                        && convertData.getFileConvertTime() != null &&
                                convertData.getFileConvertTime().compareTo(BigDecimal.ZERO) > 0).count();
                        if (count <= 0) {
                            convertTime = BigDecimal.ZERO;
                        } else {
                            convertTime = backgroundConvertData.stream().filter(convertData ->
                                    MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus,-queryDTO.getTimeZone()))
                                            && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus.plus(1L, ChronoUnit.HOURS),-queryDTO.getTimeZone()))
                                            && convertData.getFileConvertTime() != null &&
                                    convertData.getFileConvertTime().compareTo(BigDecimal.ZERO) > 0)
                                    .map(BackgroundConvertData::getFileConvertTime).reduce(BigDecimal.ZERO, BigDecimal::add)
                                    .divide(BigDecimal.valueOf(count), 2, BigDecimal.ROUND_HALF_UP);
                        }
                        dataDTOS.add(DashboardDetailedDataDTO.builder()
                                .xData(minus.toString())
                                .yData(convertTime.stripTrailingZeros().toPlainString()).build());
                    }
                } else {
                    for (int i = 0; i < days; i++) {
                        // 减去 i 天时间
                        LocalDate minus = endDateTime.minus(i, ChronoUnit.DAYS);
                        long count = backgroundConvertData.stream().filter(convertData ->
                                MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus,-queryDTO.getTimeZone()))
                                && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus.plusDays(1),-queryDTO.getTimeZone()))
                                        && convertData.getFileConvertTime() != null &&
                                convertData.getFileConvertTime().compareTo(BigDecimal.ZERO) > 0).count();
                        if (count <= 0) {
                            convertTime = BigDecimal.ZERO;
                        } else {
                            convertTime = backgroundConvertData.stream().filter(convertData ->
                                    MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus,-queryDTO.getTimeZone()))
                                            && !MyLocalDateUtil.localDataCompareTo(convertData.getSyncTime(), MyLocalDateUtil.timeZoneConversion(minus.plusDays(1),-queryDTO.getTimeZone()))
                                            && convertData.getFileConvertTime() != null &&
                                    convertData.getFileConvertTime().compareTo(BigDecimal.ZERO) > 0)
                                    .map(BackgroundConvertData::getFileConvertTime).reduce(BigDecimal.ZERO, BigDecimal::add)
                                    .divide(BigDecimal.valueOf(count), 2, BigDecimal.ROUND_HALF_UP);
                        }
                        dataDTOS.add(DashboardDetailedDataDTO.builder()
                                .xData(minus.toString())
                                .yData(convertTime.stripTrailingZeros().toPlainString()).build());
                    }
                }
                break;
            default:
                throw new CommonException(ResponseEnum.SYSTEM_ERROR);
        }

        // 计算平均值，确定单位
        BigDecimal average = BigDecimal.ZERO;
        for (DashboardDetailedDataDTO dataDTO : dataDTOS) {
            average = average.add(new BigDecimal(dataDTO.getYData()));
        }
        average = average.divide(new BigDecimal(dataDTOS.size()), 2, BigDecimal.ROUND_HALF_UP);
        Integer flag = null;
        // 平均值大于秒
        if (average.compareTo(new BigDecimal("1000")) >= 0) {
            flag = 1;
        }
        // 平均值大于分钟
        if (average.compareTo(new BigDecimal("60000")) >= 0) {
            flag = 2;
        }
        // 转换
        if (null != flag) {
            for (DashboardDetailedDataDTO dataDTO : dataDTOS) {
                dataDTO.setYData(TimeConvert(new BigDecimal(dataDTO.getYData()), flag));
            }
        } else {
            for (DashboardDetailedDataDTO dataDTO : dataDTOS) {
                dataDTO.setYData(dataDTO.getYData() + "ms");
            }
        }
        Collections.reverse(dataDTOS);
        return dataDTOS;
    }

    @Override
    public void dashboardExport(DashboardQueryDTO queryDTO, HttpServletResponse response) {
        List<DashboardDetailedDataDTO> successfulRequestDetailedData = this.getSuccessfulRequestDetailedData(queryDTO);
        List<DashboardDetailedDataDTO> invalidRequestDetailedData = this.getInvalidRequestDetailedData(queryDTO);
        List<DashboardDetailedDataDTO> errorRequestDetailedData = this.getErrorRequestDetailedData(queryDTO);
        List<DashboardDetailedDataDTO> errorRatioDetailedData = this.getErrorRatioDetailedData(queryDTO);
        List<DashboardDetailedDataDTO> averageProcessTimeDetailedData = this.getAverageProcessTimeDetailedData(queryDTO);
        List<DashboardExportObject> dashboardExportObjects = new ArrayList<>();
        boolean hourOrDate = (queryDTO.getQueryType() == 0 && queryDTO.getDays() == 1) || (queryDTO.getQueryType() == 1 && queryDTO.getStartDateTime().equals(queryDTO.getEndDateTime()));
        String time;
        for (int i = 0; i < successfulRequestDetailedData.size(); i++) {
            time = successfulRequestDetailedData.get(i).getXData();
            if (hourOrDate) {
                time = time.replaceFirst("T", " ");
            }
            dashboardExportObjects.add(DashboardExportObject.builder()
                    .time(time)
                    .successfulRequests(successfulRequestDetailedData.get(i).getYData())
                    .invalidRequest(invalidRequestDetailedData.get(i).getYData())
                    .errorRequests(errorRequestDetailedData.get(i).getYData())
                    .errorRatio(errorRatioDetailedData.get(i).getYData())
                    .averageProcessTime(averageProcessTimeDetailedData.get(i).getYData())
                    .build());
        }

        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        List<List<String>> listHand = new ArrayList<>();
        List<String> head0 = new ArrayList<>();
        List<String> head1 = new ArrayList<>();
        List<String> head2 = new ArrayList<>();
        List<String> head3 = new ArrayList<>();
        List<String> head4 = new ArrayList<>();
        List<String> head5 = new ArrayList<>();

        if (queryDTO.getIsCN()){
            // 判断是Hour还是Date
            head0.add(hourOrDate ? "小时" : "天");
            head1.add("成功请求数");
            head2.add("无效请求数");
            head3.add("错误请求数");
            head4.add("错误率");
            head5.add("平均处理时间");
        }else {
            // 判断是Hour还是Date
            head0.add(hourOrDate ? "Hour" : "Date");
            head1.add("Successful Requests");
            head2.add("Invalid Requests");
            head3.add("Error Requests");
            head4.add("Error Ratio");
            head5.add("Average Process Time");

        }
        listHand.add(head0);
        listHand.add(head1);
        listHand.add(head2);
        listHand.add(head3);
        listHand.add(head4);
        listHand.add(head5);

        // 查询类型
        Integer queryType = queryDTO.getQueryType();
        if (ObjectUtil.isEmpty(queryDTO.getTimeZone())){
            queryDTO.setTimeZone(0);
        }

        LocalDateTime startDate = LocalDateTime.now();
        LocalDateTime endDate = LocalDateTime.now();
        if (queryType == 0) {
            Integer days = queryDTO.getDays();
            if (days > 30) {
                days = 30;
                queryDTO.setDays(30);
            } else if (days < 1) {
                days = 1;
                queryDTO.setDays(1);
            }
            startDate = LocalDateTime.now();
            endDate = startDate.minus(days, ChronoUnit.DAYS);
        } else if (queryType == 1) {
            LocalDate startDateTime = queryDTO.getStartDateTime();
            LocalDate endDateTime = queryDTO.getEndDateTime();

            if (startDateTime.isAfter(endDateTime)) {
                // 查询参数异常 开始时间大于结束时间
                throw new CommonException(BackgroundErrorEnum.QUEUE_PARAMETER_ERROR);
            } else if (endDateTime.isAfter(MyLocalDateUtil.timeZoneConversion(LocalDateTime.now(), queryDTO.getTimeZone()).toLocalDate())) {
                throw new CommonException(BackgroundErrorEnum.QUEUE_PARAMETER_ERROR);
            }
            // 开始时间等于结束时间
            endDateTime = endDateTime.plusDays(1);
            startDate = MyLocalDateUtil.timeZoneConversion(startDateTime, -queryDTO.getTimeZone());
            endDate = MyLocalDateUtil.timeZoneConversion(endDateTime, -queryDTO.getTimeZone());
        }

        try {
            // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
            String fileName = URLEncoder.encode("Dashboard表单", "UTF-8").replaceAll("\\+", "%20");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
            ServletOutputStream outputStream = response.getOutputStream();
            ExcelWriter excelWriter = EasyExcel.write(outputStream)
                    .registerWriteHandler(new ExcelWidthStyleStrategy())
                    .autoCloseStream(Boolean.FALSE)
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    .build();

            excelWriter.write(dashboardExportObjects, EasyExcel.writerSheet("Dashboard").head(listHand).build());

            List<ProjectQueryVO> projectQuery = backgroundUserProjectService.getProjectQuery(TokenUtil.getRequestHeader().getTenantId());
            List<ToolQueryVO> toolQueryS = toolsService.getToolQueryS();
            LocalDateTime finalStartDate = startDate;
            LocalDateTime finalEndDate = endDate;
            projectQuery.forEach(projectQueryVO -> {
                // 创建 WriteSheet 对象，指定 sheet 名称
                WriteSheet writeSheet = EasyExcel.writerSheet(projectQueryVO.getProjectName()).build();
                List<DashExportXlsxDTO> dashExportXlsxDTOS = new ArrayList<>();
                if (queryDTO.getIsCN()){
                    dashExportXlsxDTOS.add(new DashExportXlsxDTO("时间: " + formatter.format(finalEndDate) + "-" + formatter.format(finalStartDate), "", "", "", "", ""));
                    dashExportXlsxDTOS.add(new DashExportXlsxDTO("功能", "成功", "无效", "失败", "失败率", "平均处理时长"));
                }else {
                    dashExportXlsxDTOS.add(new DashExportXlsxDTO("Date: " + formatter.format(finalEndDate) + "-" + formatter.format(finalStartDate), "", "", "", "", ""));
                    dashExportXlsxDTOS.add(new DashExportXlsxDTO("Features", "Successful Requests", "Invalid Requests", "Error Requests", "Error Ratio", "Average Process Time"));
                }


                toolQueryS.forEach(toolQueryVO -> {
                    DashboardQueryDTO toolQueryDTO = new DashboardQueryDTO();
                    BeanUtils.copyProperties(queryDTO, toolQueryDTO);
                    toolQueryDTO.setProjectId(projectQueryVO.getProjectId());
                    toolQueryDTO.setToolId(toolQueryVO.getId());
                    AnalysisDataDTO analysisData = this.getAnalysisData(toolQueryDTO);
                    dashExportXlsxDTOS.add(new DashExportXlsxDTO(queryDTO.getIsCN() ? toolQueryVO.getCnName() : toolQueryVO.getEnName(),
                            analysisData.getSuccessfulRequest().toString(),
                            analysisData.getInvalidRequest().toString(),
                            analysisData.getErrorRequest().toString(),
                            analysisData.getErrorRatio(),
                            analysisData.getAverageProcessTime()));
                });
                DashboardQueryDTO toolQueryDTO = new DashboardQueryDTO();
                BeanUtils.copyProperties(queryDTO, toolQueryDTO);
                toolQueryDTO.setProjectId(projectQueryVO.getProjectId());
                AnalysisDataDTO analysisData = this.getAnalysisData(toolQueryDTO);

                dashExportXlsxDTOS.add(new DashExportXlsxDTO("", "", "", "", "", ""));
                if (queryDTO.getIsCN()){
                    dashExportXlsxDTOS.add(new DashExportXlsxDTO("累计",
                            analysisData.getSuccessfulRequest().toString(),
                            analysisData.getInvalidRequest().toString(),
                            analysisData.getErrorRequest().toString(),
                            analysisData.getErrorRatio(),
                            analysisData.getAverageProcessTime()));
                    dashExportXlsxDTOS.add(new DashExportXlsxDTO("注：数据为所选期间内的累积数据。", "", "", "", "", ""));
                }else {
                    dashExportXlsxDTOS.add(new DashExportXlsxDTO("Total",
                            analysisData.getSuccessfulRequest().toString(),
                            analysisData.getInvalidRequest().toString(),
                            analysisData.getErrorRequest().toString(),
                            analysisData.getErrorRatio(),
                            analysisData.getAverageProcessTime()));
                    dashExportXlsxDTOS.add(new DashExportXlsxDTO("Note: The count is the accumulated data within the time range.", "", "", "", "", ""));
                }

                excelWriter.write(dashExportXlsxDTOS, writeSheet);
            });
            excelWriter.finish();
        } catch (Exception e) {
            // 重置response
            response.reset();
            response.setContentType("application/json");
            response.setCharacterEncoding("utf-8");
            Map<String, String> map = MapUtils.newHashMap();
            map.put("status", "failure");
            map.put("message", "下载文件失败" + e.getMessage());
            try {
                response.getWriter().println(JSON.toJSONString(map));
            } catch (IOException ioException) {
                ioException.printStackTrace();
            }
        }
    }

    @Override
    @CacheEvict(cacheNames = RedisConstantServer.DASHBOARD + "*", cacheManager = "myCacheManager", allEntries = true)
    public void dashboardDataSync(List<DashboardStatisticsDTO> list) {
        assert list != null : "Dashboard同步数据为空";
        list.forEach(item ->{
            item.setUserId(userService.getUserId(item.getTenantId()));
        });
        final String DASHBOARD_SYNC_KEY = "DashboardSyncKey:" + list.get(0).getSyncTime();
        Boolean b = redisTemplate.hasKey(DASHBOARD_SYNC_KEY);
        assert b != null;
        if (!b) {
            log.info("开始数据同步");
            redisTemplate.opsForValue().setIfAbsent(DASHBOARD_SYNC_KEY, "", 59, TimeUnit.MINUTES);
            try {
                this.baseMapper.insertList(list);
            } catch (Exception e) {
                redisTemplate.delete(DASHBOARD_SYNC_KEY);
                throw e;
            }
        } else {
            log.error("当前时间节点已同步过：{}" + DASHBOARD_SYNC_KEY);
        }
    }

    @Override
    public void refresh(Long tenantId) {
        String cacheKey = ":DashboardQueryDTO(tenantId=" +
                tenantId +
                ",*";
        String analysisData = RedisConstantServer.DASHBOARD_ANALYSIS_DATA + cacheKey;
        String successData = RedisConstantServer.DASHBOARD_SUCCESS_DATA + cacheKey;
        String failData = RedisConstantServer.DASHBOARD_FAIL_DATA + cacheKey;
        String errorRatioData = RedisConstantServer.DASHBOARD_ERROR_RATIO_DATA + cacheKey;
        String averageProcessData = RedisConstantServer.DASHBOARD_AVERAGE_PROCESS_DATA + cacheKey;

        redisTemplate.delete(Objects.requireNonNull(redisTemplate.keys(analysisData)));
        redisTemplate.delete(Objects.requireNonNull(redisTemplate.keys(successData)));
        redisTemplate.delete(Objects.requireNonNull(redisTemplate.keys(failData)));
        redisTemplate.delete(Objects.requireNonNull(redisTemplate.keys(errorRatioData)));
        redisTemplate.delete(Objects.requireNonNull(redisTemplate.keys(averageProcessData)));
        log.info("删除缓存成功，tenantId：{}", tenantId);
        // TODO 强制刷新去同步转档数据库数据
    }


    /**
     * 获取Dashboard 查询使用的LambdaQueryWrapper
     *
     * @param queryDTO 查询参数
     * @return Dashboard 查询使用的LambdaQueryWrapper
     */
    private LambdaQueryWrapper<BackgroundConvertData> getDashboardQueryWrapper(DashboardQueryDTO queryDTO) {
        LambdaQueryWrapper<BackgroundConvertData> queryWrapper = new LambdaQueryWrapper<>();
        // 查询类型
        Integer queryType = queryDTO.getQueryType();
        // 如果时区未传，默认0时区
        if (ObjectUtil.isEmpty(queryDTO.getTimeZone())){
            queryDTO.setTimeZone(0);
        }
        log.info("本次查询AnalysisData参数：{}", queryDTO);
        /* 如果时间范围查询为系统定义 */
        if (queryType == 0) {
            Integer days = queryDTO.getDays();
            if (days > 30) {
                days = 30;
                queryDTO.setDays(30);
            } else if (days < 1) {
                days = 1;
                queryDTO.setDays(1);
            }
            LocalDateTime nowDateTime = LocalDateTime.now();
            LocalDateTime minus = nowDateTime.minus(days, ChronoUnit.DAYS);
            queryWrapper.ge(BackgroundConvertData::getSyncTime, minus)
                    .le(BackgroundConvertData::getSyncTime,nowDateTime);
        } else if (queryType == 1) {
            LocalDate startDateTime = queryDTO.getStartDateTime();
            LocalDate endDateTime = queryDTO.getEndDateTime();

            if (startDateTime.isAfter(endDateTime)) {
                // 查询参数异常 开始时间大于结束时间
                throw new CommonException(BackgroundErrorEnum.QUEUE_PARAMETER_ERROR);
            } else if (endDateTime.isAfter(MyLocalDateUtil.timeZoneConversion(LocalDateTime.now(), queryDTO.getTimeZone()).toLocalDate())) {
                throw new CommonException(BackgroundErrorEnum.QUEUE_PARAMETER_ERROR);
            }
//            else if (startDateTime.equals(endDateTime)) {
                // 开始时间等于结束时间
                endDateTime = endDateTime.plusDays(1);
//            }
//            else if (ChronoUnit.DAYS.between(startDateTime, endDateTime) > 30) {
//                // 开始时间和结束时间大于30天
//                startDateTime = endDateTime.minus(30, ChronoUnit.DAYS);
//                queryDTO.setStartDateTime(endDateTime.minus(30, ChronoUnit.DAYS));
//            }
            queryWrapper.ge(BackgroundConvertData::getSyncTime, MyLocalDateUtil.timeZoneConversion(startDateTime, -queryDTO.getTimeZone()))
                    .le(BackgroundConvertData::getSyncTime, MyLocalDateUtil.timeZoneConversion(endDateTime, -queryDTO.getTimeZone()));
        } else {
            throw new CommonException(BackgroundErrorEnum.ERROR_INNER);
        }
        // 如果条件 projectId不为空则添加查询
        if (queryDTO.getProjectId() != null) {
            queryWrapper.eq(BackgroundConvertData::getProjectId, queryDTO.getProjectId());
        }
        // 如果条件 toolId不为空则添加查询
        if (queryDTO.getToolId() != null) {
            queryWrapper.eq(BackgroundConvertData::getToolId, queryDTO.getToolId());
        }
        queryWrapper.eq(BackgroundConvertData::getTenantId, queryDTO.getTenantId());
        return queryWrapper;
    }

}
