package cn.kdan.compdf.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.kdan.compdf.config.ExcelWidthStyleStrategy;
import cn.kdan.compdf.constant.AssetTypeEnum;
import cn.kdan.compdf.constant.UserBalanceConstant;
import cn.kdan.compdf.dto.AddFreePlanDTO;
import cn.kdan.compdf.dto.Asset;
import cn.kdan.compdf.dto.PlanStatisticsDTO;
import cn.kdan.compdf.entity.BackgroundUserBalance;
import cn.kdan.compdf.entity.BackgroundUserRemain;
import cn.kdan.compdf.enums.DateTImeFormatEnum;
import cn.kdan.compdf.enums.ResponseEnum;
import cn.kdan.compdf.exception.BusinessException;
import cn.kdan.compdf.mapper.BackgroundUserBalanceMapper;
import cn.kdan.compdf.service.BackgroundUserBalanceService;
import cn.kdan.compdf.service.BackgroundUserRemainService;
import cn.kdan.compdf.service.BackgroundUserService;
import cn.kdan.compdf.utils.MyLocalDateUtil;
import cn.kdan.compdf.utils.TokenUtil;
import cn.kdan.compdf.vo.BackgroundUserBalanceVO;
import cn.kdan.compdf.vo.BackgroundUserPackageBalanceVO;
import cn.kdan.compdf.vo.BillVO;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.util.MapUtils;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.TemporalAdjusters;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2022-10-14
 */
@Service
@Slf4j
public class BackgroundUserBalanceServiceImpl extends ServiceImpl<BackgroundUserBalanceMapper, BackgroundUserBalance> implements BackgroundUserBalanceService {


    private final StringRedisTemplate redisTemplate;

    private final BackgroundUserService userService;
    private final BackgroundUserRemainService backgroundUserRemainService;
    @Value("${free-100-tenantId}")
    private Integer FREE_100_TENANT_ID;

    public BackgroundUserBalanceServiceImpl(StringRedisTemplate redisTemplate, BackgroundUserService userService, BackgroundUserRemainService backgroundUserRemainService) {
        this.redisTemplate = redisTemplate;
        this.userService = userService;
        this.backgroundUserRemainService = backgroundUserRemainService;
    }

    @Override
    public IPage<BackgroundUserBalance> page(BackgroundUserBalance query) {
        return this.page(new Page<>(query.getPage(), query.getSize()), Wrappers.query(query));
    }

    /**
     * 获取账户套餐余额
     *
     * @param userId 用户id
     * @return 账户套餐余额
     */
    @Override
    public BackgroundUserPackageBalanceVO getPackageBalance(Long userId) {
//        LambdaQueryWrapper<BackgroundUserRemain> eq = new LambdaQueryWrapper<BackgroundUserRemain>()
//                .eq(BackgroundUserRemain::getUserId, userId);
//        BackgroundUserRemain backgroundUserRemain = backgroundUserRemainService.getOne(eq);
        List<BackgroundUserRemain> userRemains = backgroundUserRemainService.getUserRemainByUserId(userId);
        int usedFiles = 0;
        int remainingFiles = 0;
        for (BackgroundUserRemain userRemain : userRemains) {
            usedFiles = usedFiles + userRemain.getUsedFiles();
            remainingFiles = remainingFiles + userRemain.getTotalFiles();
        }
        return BackgroundUserPackageBalanceVO.builder()
                .usedFiles(usedFiles).remainingFiles(remainingFiles).build();
    }

    /**
     * 获取账户消费充值记录
     *
     * @param userId 用户id
     * @param page   当前页
     * @param size   当前页行数
     * @return 账户消费充值记录
     */
    @Override
    public IPage<BackgroundUserBalanceVO> getBalanceRecordList(Long userId, Integer page, Integer size, Integer timeZone) {
        LambdaQueryWrapper<BackgroundUserBalance> eq = new LambdaQueryWrapper<BackgroundUserBalance>()
                .eq(BackgroundUserBalance::getUserId, userId)
                .orderByDesc(BackgroundUserBalance::getId);
        Page<BackgroundUserBalance> backgroundUserBalancePage = this.baseMapper.selectPage(new Page<>(page, size), eq);
        // 处理返回数据
        Page<BackgroundUserBalanceVO> backgroundUserBalanceVOPage = new Page<>();
        backgroundUserBalanceVOPage.setCurrent(backgroundUserBalancePage.getCurrent());
        backgroundUserBalanceVOPage.setPages(backgroundUserBalancePage.getPages());
        backgroundUserBalanceVOPage.setSize(backgroundUserBalancePage.getSize());
        backgroundUserBalanceVOPage.setTotal(backgroundUserBalancePage.getTotal());
        List<BackgroundUserBalanceVO> records = Lists.newArrayList();
        if (CollectionUtil.isNotEmpty(backgroundUserBalancePage.getRecords())) {
            backgroundUserBalancePage.getRecords().forEach(c -> {
//                c.setDate(MyLocalDateUtil.timeZoneConversion(c.getDate(), timeZone));
                BackgroundUserBalanceVO backgroundUserBalanceVO = new BackgroundUserBalanceVO();
                BeanUtil.copyProperties(c, backgroundUserBalanceVO);
                backgroundUserBalanceVO.setDate(DateUtil.format(MyLocalDateUtil.timeZoneConversion(c.getDate(),timeZone), DateTImeFormatEnum.DATE_TIME_SECOND));
                if (c.getChangeType() == 1) {
                    backgroundUserBalanceVO.setBalanceChange("+" + c.getBalanceChange());
                } else {
                    backgroundUserBalanceVO.setBalanceChange("-" + c.getBalanceChange());
                }
                records.add(backgroundUserBalanceVO);
            });
        }
        backgroundUserBalanceVOPage.setRecords(records);
        return backgroundUserBalanceVOPage;
    }

    /**
     * 账户消费充值记录导出
     *
     * @param userId   用户id
     * @param response response
     */
    @Override
    public void exportBalanceRecordList(Integer timeZone,Integer isDecs, Long userId,Boolean isCN, HttpServletResponse response) {
        LambdaQueryWrapper<BackgroundUserBalance> eq = new LambdaQueryWrapper<BackgroundUserBalance>()
                .eq(BackgroundUserBalance::getUserId, userId)
                .orderByDesc(BackgroundUserBalance::getId);
        List<BackgroundUserBalance> backgroundUserBalanceList = this.baseMapper.selectList(eq);
        // 处理返回数据
        List<BackgroundUserBalanceVO> backgroundUserBalanceVOList = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(backgroundUserBalanceList)) {
            backgroundUserBalanceList.forEach(c -> {
                BackgroundUserBalanceVO backgroundUserBalanceVO = new BackgroundUserBalanceVO();
                BeanUtil.copyProperties(c, backgroundUserBalanceVO);
                backgroundUserBalanceVO.setDate(DateUtil.format(MyLocalDateUtil.timeZoneConversion(c.getDate(),timeZone), DateTImeFormatEnum.DATE_TIME));
                if (c.getChangeType() == 1) {
                    backgroundUserBalanceVO.setBalanceChange("+" + c.getBalanceChange());
                } else {
                    backgroundUserBalanceVO.setBalanceChange("-" + c.getBalanceChange());
                }
                backgroundUserBalanceVOList.add(backgroundUserBalanceVO);
            });
        }
        if (isDecs == 0) {
            Collections.reverse(backgroundUserBalanceVOList);
        }
        // 写入响应导出
        try {
            String fileName = URLEncoder.encode("Processing Files Balance (" + DateUtil.date().toString().replaceAll(":", "-") + ")", "UTF-8").replaceAll("\\+", "%20");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");

            List<List<String>> listHand = new ArrayList<>();
            List<String> head0 = new ArrayList<>();
            List<String> head1 = new ArrayList<>();
            List<String> head2 = new ArrayList<>();
            List<String> head3 = new ArrayList<>();
            List<String> head4 = new ArrayList<>();

            if (isCN){
                head0.add("日期");
                head1.add("描述");
                head2.add("余额变动");
                head3.add("剩余文档数量");
                head4.add("id");
                backgroundUserBalanceVOList.forEach(userBalance -> {
                    switch (userBalance.getDescription()){
                        case "Free Files":
                            userBalance.setDescription("免费文档数");
                            break;
                        case "Monthly Subscription Files":
                            userBalance.setDescription("月订阅文档数");
                            break;
                        case "Annually Subscription Files":
                            userBalance.setDescription("年订阅文档数");
                            break;
                        case "Files Expired":
                            userBalance.setDescription("过期文档数");
                            break;
                        case "Package Files":
                            userBalance.setDescription("套餐包文档数");
                            break;
                        case "Files Consumed":
                        case "Files consumed":
                            userBalance.setDescription("已消耗文档数");
                            break;
                    }
                    if (userBalance.getDescription().contains("Package")) {
                        userBalance.setDescription("套餐包文档数");
                    } else if (userBalance.getDescription().contains("Monthly Subscription")) {
                        userBalance.setDescription("月订阅文档数");
                    } else if (userBalance.getDescription().contains("Free 100 Files")) {
                        userBalance.setDescription("免费文档数");
                    }
                });
            }else {
                // 判断是Hour还是Date
                head0.add("Date");
                head1.add("Description");
                head2.add("Balance Change");
                head3.add("Remaining Files");
                head4.add("id");
            }
            listHand.add(head0);
            listHand.add(head1);
            listHand.add(head2);
            listHand.add(head3);
            listHand.add(head4);

            EasyExcel.write(response.getOutputStream())
                    .head(listHand)
                    .registerWriteHandler(new ExcelWidthStyleStrategy())
                    .autoCloseStream(Boolean.FALSE)
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    .sheet("Processing Files Balance")
                    .doWrite(backgroundUserBalanceVOList);

        } catch (Exception e) {
            // 重置response
            response.reset();
            response.setContentType("application/json");
            response.setCharacterEncoding("utf-8");
            Map<String, String> map = MapUtils.newHashMap();
            map.put("status", "failure");
            map.put("message", "下载文件失败" + e.getMessage());
            try {
                response.getWriter().println(JSON.toJSONString(map));
            } catch (IOException ioException) {
                ioException.printStackTrace();
            }
        }
    }

    @Override
    public void planSync(List<PlanStatisticsDTO> list) {
        assert list != null : "Dashboard同步数据为空";
        final String PLAN_SYNC_KEY = "planSyncKey:" + list.get(0).getSyncTime();
        Boolean b = redisTemplate.hasKey(PLAN_SYNC_KEY);
        assert b != null;
        if (!b) {
            log.info("开始数据同步");
            redisTemplate.opsForValue().setIfAbsent(PLAN_SYNC_KEY, "", 1, TimeUnit.DAYS);
            try {
                List<BackgroundUserBalance> userBalanceList = new ArrayList<>();
                list.forEach(item -> {
                    Long userId = userService.getUserId(item.getTenantId());
                    // 最新的plan剩余额度
                    Integer newestPlanRemainingFiles = getNewestPlanRemainingFiles(item.getTenantId());
                    BackgroundUserBalance backgroundUserBalance = new BackgroundUserBalance();
                    backgroundUserBalance.setUserId(userId);
                    backgroundUserBalance.setTenantId(item.getTenantId());
                    backgroundUserBalance.setDate(LocalDateTime.now());
                    backgroundUserBalance.setDescriptionType(9);
                    backgroundUserBalance.setDescription(UserBalanceConstant.DESCRIPTION_FILES_CONSUMED);  // TODO 枚举值
                    backgroundUserBalance.setChangeType(2);
                    backgroundUserBalance.setRemainingFiles((int) (newestPlanRemainingFiles - item.getPlan()));
                    backgroundUserBalance.setBalanceChange(item.getPlan().intValue());
                    userBalanceList.add(backgroundUserBalance);
                });
                this.saveBatch(userBalanceList);
            } catch (Exception e) {
                redisTemplate.delete(PLAN_SYNC_KEY);
                throw e;
            }
        } else {
            log.error("当前时间节点已同步过：{}" + PLAN_SYNC_KEY);
        }
    }

    @Override
    public void freePlanSync(AddFreePlanDTO freePlanDTO) {
        Integer freeAsset = freePlanDTO.getFreeAsset();
        List<Asset> assets = freePlanDTO.getAssets();
        for (Asset asset : assets) {
            // 最新的plan剩余额度
            Integer newestPlanRemainingFiles = getNewestPlanRemainingFiles(asset.getTenantId());
            BackgroundUserBalance userBalance = new BackgroundUserBalance();
            userBalance.setTenantId(asset.getTenantId());
            userBalance.setUserId(userService.getUserId(asset.getTenantId()));
            userBalance.setDate(LocalDateTime.now());
            userBalance.setDescriptionType(9);
            userBalance.setDescription(UserBalanceConstant.DESCRIPTION_FILES_EXPIRED); // TODO 枚举值
            userBalance.setRemainingFiles(newestPlanRemainingFiles - asset.getAsset());
            userBalance.setBalanceChange(asset.getAsset());
            userBalance.setChangeType(3);

            this.save(userBalance);

            newestPlanRemainingFiles = getNewestPlanRemainingFiles(asset.getTenantId());
            if (asset.getTenantId() >= FREE_100_TENANT_ID) {
                userBalance.setId(null);
                userBalance.setDate(userBalance.getDate().plusSeconds(1L));
                userBalance.setDescriptionType(3);
                userBalance.setDescription(UserBalanceConstant.DESCRIPTION_FREE); // TODO 枚举值
                userBalance.setRemainingFiles(newestPlanRemainingFiles + 1000);
                userBalance.setBalanceChange(1000);
                userBalance.setChangeType(1);
                this.save(userBalance);
            }else {
                userBalance.setId(null);
                userBalance.setDate(userBalance.getDate().plusSeconds(1L));
                userBalance.setDescriptionType(3);
                userBalance.setDescription(UserBalanceConstant.DESCRIPTION_FREE); // TODO 枚举值
                userBalance.setRemainingFiles(newestPlanRemainingFiles + freeAsset);
                userBalance.setBalanceChange(freeAsset);
                userBalance.setChangeType(1);
                this.save(userBalance);
            }
        }
        List<Asset> cleanFreeAssets = freePlanDTO.getCleanFreeAssets();
        for (Asset cleanFreeAsset : cleanFreeAssets) {
            // 最新的plan剩余额度
            Integer newestPlanRemainingFiles = getNewestPlanRemainingFiles(cleanFreeAsset.getTenantId());
            BackgroundUserBalance userBalance = new BackgroundUserBalance();
            userBalance.setTenantId(cleanFreeAsset.getTenantId());
            userBalance.setUserId(userService.getUserId(cleanFreeAsset.getTenantId()));
            userBalance.setDate(LocalDateTime.now());
            userBalance.setDescriptionType(9);
            userBalance.setDescription(UserBalanceConstant.DESCRIPTION_FILES_CONSUMED); // TODO 枚举值
            userBalance.setRemainingFiles(newestPlanRemainingFiles - cleanFreeAsset.getAsset());
            userBalance.setBalanceChange(cleanFreeAsset.getAsset());
            userBalance.setChangeType(3);

            this.save(userBalance);

            Integer newestPlanAddFreeNum = this.baseMapper.selectNewestPlanAddFreeNum(cleanFreeAsset.getTenantId());
            backgroundUserRemainService.cleanFreeAssetTotal(cleanFreeAsset.getTenantId(), newestPlanAddFreeNum);

        }

    }

    @Override
    public void initUserBalance(Long userId,Long tenantId, Integer assets){
        BackgroundUserBalance userBalance = new BackgroundUserBalance();
        userBalance.setUserId(userId);
        userBalance.setTenantId(tenantId);
        userBalance.setDate(LocalDateTime.now());
        userBalance.setDescriptionType(UserBalanceConstant.DESCRIPTION_TYPE_FREE);
        userBalance.setDescription(UserBalanceConstant.DESCRIPTION_FREE);
        userBalance.setChangeType(UserBalanceConstant.CHANGE_TYPE_ADD);
        userBalance.setBalanceChange(assets);
        userBalance.setRemainingFiles(assets);

        this.baseMapper.insert(userBalance);
    }

    @Override
    public List<BillVO> getBill(Long userId) {
        List<BackgroundUserRemain> userRemains = backgroundUserRemainService.getUserRemainByUserId(userId);
        return userRemains.stream().map(userRemain -> {
            BillVO billVO = new BillVO();
            billVO.setAssetType(userRemain.getAssetType());
            billVO.setTotalFiles(userRemain.getTotalFiles());
            //  资产类型enum
            switch (AssetTypeEnum.getByValue(userRemain.getAssetType())) {
                case FREE:
                    LocalDateTime userFirstLogin = userService.getUSerFirstLogin(TokenUtil.getRequestHeader().getTenantId());
                    billVO.setStartDate(userFirstLogin.toLocalDate().compareTo(this.getFirstDayOfMonth()) < 0  ? this.getFirstDayOfMonth() : userFirstLogin.toLocalDate() );
                    billVO.setEndDate(this.getLastDayOfMonth());
                    break;
                case SUBSCRIPTION:
                    billVO.setStartDate(ObjectUtil.isNotEmpty(userRemain.getStartDate()) ? userRemain.getStartDate().toLocalDate() : null);
                    billVO.setEndDate(ObjectUtil.isNotEmpty(userRemain.getEndDate()) ? userRemain.getEndDate().toLocalDate() : null);
                    billVO.setCycle(userRemain.getCycle());
                    break;
                case PACKAGE: break;
                default: throw new BusinessException(ResponseEnum.ERROR_ASSET_TYPE_IS_NOT_SUPPORTED_YET);
            }
            return billVO;
        }).collect(Collectors.toList());
    }


    /**
     * 获取最新的plan资产余额
     *
     * @param tenantId tenantId
     * @return 最新的plan资产余额
     */
    private Integer getNewestPlanRemainingFiles(Long tenantId) {
        Integer newestPlanRemainingFiles = this.baseMapper.selectNewestPlanRemainingFiles(tenantId);
        if (ObjectUtil.isEmpty(newestPlanRemainingFiles)) {
            newestPlanRemainingFiles = 0;
        }
        return newestPlanRemainingFiles;
    }

    private LocalDate getFirstDayOfMonth() {
        return LocalDate.now().with(TemporalAdjusters.firstDayOfMonth());
    }

    private LocalDate getLastDayOfMonth() {
        return LocalDate.now().with(TemporalAdjusters.lastDayOfMonth());
    }
}
