package cn.kdan.compdf.service.impl;

import cn.kdan.compdf.constant.MyConstant;
import cn.kdan.compdf.entity.OrderCashFlow;
import cn.kdan.compdf.entity.OrdersGoods;
import cn.kdan.compdf.exception.BusinessException;
import cn.kdan.compdf.mapper.OrderCashFlowMapper;
import cn.kdan.compdf.service.OrderCashFlowService;
import cn.kdan.compdf.service.OrdersGoodsService;
import cn.kdan.compdf.vo.OrderBillVo;
import cn.kdan.compdf.vo.OrderBillingInfoVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2023-05-25
 */
@Service
@RequiredArgsConstructor
public class OrderCashFlowServiceImpl extends ServiceImpl<OrderCashFlowMapper, OrderCashFlow> implements OrderCashFlowService {

    private final OrdersGoodsService ordersGoodsService;

    /**
     * 获取流水账单
     *
     * @param orderIds orderIds
     * @return OrderBillVo
     */
    @Override
    public List<OrderBillVo> getOrderCashFlowByOrderId(List<Integer> orderIds, Integer flag) {
        List<OrderCashFlow> orderCashFlows = new ArrayList<>();
        orderIds = orderIds.stream().distinct().collect(Collectors.toList());
        if (CollectionUtils.isEmpty(orderIds)){
            return new ArrayList<>();
        }
        List<OrdersGoods> ordersGoods = ordersGoodsService.getBaseMapper().selectList(new LambdaQueryWrapper<OrdersGoods>()
                .select(OrdersGoods::getId)
                .in(OrdersGoods::getOrderId, orderIds));
        List<Integer> ordersGoodsIds = ordersGoods.stream().map(OrdersGoods::getId).collect(Collectors.toList());

        if (MyConstant.ORDER_SUCCESS.equals(flag)){
            List<OrderCashFlow> orderCashFlowsSuccess = this.baseMapper.selectList(new LambdaQueryWrapper<OrderCashFlow>()
                    .eq(OrderCashFlow::getDelFlag, 0)
                    .in(OrderCashFlow::getOrderGoodsId, ordersGoodsIds)
                    .orderByDesc(OrderCashFlow::getCreatedAt));
            if (!CollectionUtils.isEmpty(orderCashFlowsSuccess)) {
                orderCashFlows.addAll(orderCashFlowsSuccess);
            }
        } else if(MyConstant.ORDER_FAIL.equals(flag)) {
            // 您可以使用LIMIT和OFFSET子句来实现这一点。LIMIT子句用于指定要返回的记录数，而OFFSET子句用于指定要跳过的记录数。
            // 例如，如果您想要查询表中的所有行，但要跳过排序后的第一行，则可以使用以下语句：
            // SELECT * FROM table_name ORDER BY column_name LIMIT 18446744073709551615 OFFSET 1;
            // 在这个例子中，我们使用了一个非常大的数字作为LIMIT值，以确保返回所有剩余的行。您还想了解什么？
            orderIds.forEach(orderId ->{
                List<OrderCashFlow> orderCashFlowsFail = this.baseMapper.selectList(new LambdaQueryWrapper<OrderCashFlow>()
                        .eq(OrderCashFlow::getDelFlag, 0)
                        .eq(OrderCashFlow::getOrderGoodsId, ordersGoodsIds)
                        .orderByDesc(OrderCashFlow::getCreatedAt).last("LIMIT 18446744073709551615 OFFSET 1"));
                if (!CollectionUtils.isEmpty(orderCashFlowsFail)) {
                    orderCashFlows.addAll(orderCashFlowsFail);
                }
            });
        }
        return orderCashFlows.stream().map(this::orderCashFlowToOrderBillVo).collect(Collectors.toList());
    }


    @Override
    public void updateBillInfo(Integer orderCashFlowId, String billUrl, String billNo, String userBillStr) {
        OrderCashFlow orderCashFlow = new OrderCashFlow();
        orderCashFlow.setInvoiceNum(billNo);
        orderCashFlow.setInvoiceUrl(billUrl);
        orderCashFlow.setUserBill(userBillStr);
        orderCashFlow.setId(orderCashFlowId);
        this.baseMapper.updateById(orderCashFlow);
    }

    @Override
    public OrderBillVo getOrderCashFlowByOrderId(Integer orderCashFlowId) {
        OrderCashFlow orderCashFlow = this.baseMapper.selectOne(new LambdaQueryWrapper<OrderCashFlow>().eq(OrderCashFlow::getId, orderCashFlowId));
        return orderCashFlowToOrderBillVo(orderCashFlow);
    }

    @Override
    public OrderBillingInfoVO getOrderBillingInfo(Integer orderCashFlowId) {
        return this.baseMapper.selectOrderBillingInfo(orderCashFlowId);
    }

    /**
     * OrderCashFlow 转 OrderBillVo
     *
     * @param orderCashFlow orderCashFlow
     * @return OrderBillVo
     */
    @NotNull
    private OrderBillVo orderCashFlowToOrderBillVo(OrderCashFlow orderCashFlow) {
        OrderBillVo orderBillVo = new OrderBillVo();
        orderBillVo.setId(orderCashFlow.getId());
        orderBillVo.setOrderNo(orderCashFlow.getSerialNumber());
        orderBillVo.setPayTime(orderCashFlow.getCreatedAt());
        orderBillVo.setBillNo(orderCashFlow.getInvoiceNum());
        orderBillVo.setBillUrl(orderCashFlow.getInvoiceUrl());
        orderBillVo.setPrice(orderCashFlow.getPrice());
        return orderBillVo;
    }


}
