package cn.kdan.compdf.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.kdan.compdf.constant.MyConstant;
import cn.kdan.compdf.dto.ApplyInvoiceDTO;
import cn.kdan.compdf.dto.BillSendEmailDTO;
import cn.kdan.compdf.dto.UserBillingUpInfoDTO;
import cn.kdan.compdf.entity.UserBillingInformation;
import cn.kdan.compdf.enums.ResponseEnum;
import cn.kdan.compdf.exception.BusinessException;
import cn.kdan.compdf.exception.CommonException;
import cn.kdan.compdf.mapper.UserBillingInformationMapper;
import cn.kdan.compdf.service.OrderCashFlowService;
import cn.kdan.compdf.service.OrdersService;
import cn.kdan.compdf.service.UserBillingInformationService;
import cn.kdan.compdf.utils.TemplatesUtil;
import cn.kdan.compdf.utils.TokenUtil;
import cn.kdan.compdf.vo.*;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StreamUtils;
import org.springframework.web.client.RestTemplate;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeUtility;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.math.MathContext;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2023-05-15
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class UserBillingInformationServiceImpl extends ServiceImpl<UserBillingInformationMapper, UserBillingInformation> implements UserBillingInformationService {

    private static String invoiceHtml = null;

    @Value("${invoice.url}")
    private String invoiceUrl;

    private static final RestTemplate restTemplate = new RestTemplate();
    private final JavaMailSender javaMailSender;
    @Value("${spring.mail.username}")
    private String fromEmail;
    private final OrdersService ordersService;

    private final OrderCashFlowService orderCashFlowService;

    private static final String EMAIL_USER_NAME = "#\"userName\"";

    private final static String emailTemplate_EN = "<!DOCTYPE html>\n" +
            "<html>\n" +
            "<head>\n" +
            "<meta charset=\"utf-8\">\n" +
            "<title>HTML Editor - LDDGO.NET</title>\n" +
            "<link rel=\"stylesheet\" href=\"https://cdn.jsdelivr.net/gh/highlightjs/cdn-release@11.5.1/build/styles/default.min.css\" type=\"text/css\">\n" +
            "</head>\n" +
            "<body>\n" +
            "\n" +
            "<p>Dear "+EMAIL_USER_NAME+",</p>\n" +
            "<p>Your invoice has been sent to your email. Please check the attached PDF file.&nbsp;</p>\n" +
            "<p>Thank you for your support of ComPDFKit!</p>\n" +
            "<p><br />Thanks,<br />ComPDFKit Team<br /><a href=\"https://www.compdf.com\">https://www.compdf.com</a></p>\n" +
            "\n" +
            "<script src=\"https://cdn.jsdelivr.net/gh/highlightjs/cdn-release@11.5.1/build/highlight.min.js\" type=\"text/javascript\"></script>\n" +
            "<script type=\"text/javascript\">hljs.highlightAll();</script>\n" +
            "</body>\n" +
            "</html>";

    private final static String emailTemplate_CN = "<!DOCTYPE html>\n" +
            "<html>\n" +
            "<head>\n" +
            "<meta charset=\"utf-8\">\n" +
            "<title>HTML Editor - LDDGO.NET</title>\n" +
            "<link rel=\"stylesheet\" href=\"https://cdn.jsdelivr.net/gh/highlightjs/cdn-release@11.5.1/build/styles/default.min.css\" type=\"text/css\">\n" +
            "</head>\n" +
            "<body>\n" +
            "\n" +
            "<p>尊敬的 "+EMAIL_USER_NAME+"，</p>\n" +
            "<p>您好！您的发票已通过电子邮件发送至您的邮箱，请查收附件中的PDF文件。</p>\n" +
            "<p>感谢您对ComPDFKit的支持！</p>\n" +
            "<p><br />ComPDFKit 团队<br /><a href=\"https://www.compdf.com/zh-cn\">https://www.compdf.com/zh-cn</a></p>\n" +
            "\n" +
            "<script src=\"https://cdn.jsdelivr.net/gh/highlightjs/cdn-release@11.5.1/build/highlight.min.js\" type=\"text/javascript\"></script>\n" +
            "<script type=\"text/javascript\">hljs.highlightAll();</script>\n" +
            "</body>\n" +
            "</html>";


    static {
        try {
            invoiceHtml = new String(StreamUtils.copyToByteArray(UserBillingInformationServiceImpl.class.getClassLoader().getResourceAsStream("templates/invoice.html")), StandardCharsets.UTF_8);
        } catch (IOException e) {
            log.error("获取模板失败: ".concat(e.getMessage()), e);
        }
    }


    @Override
    public UserBillingInfoVO getUserBillingInfo(Integer compdfKitId) {
        UserBillingInformation userBillingInformation = this.baseMapper.selectOne(new LambdaQueryWrapper<UserBillingInformation>()
                .eq(UserBillingInformation::getUserId, compdfKitId));
        UserBillingInfoVO userBillingInfoVO = new UserBillingInfoVO();
        if (ObjectUtils.isEmpty(userBillingInformation)) {
            return userBillingInfoVO;
        }

        BeanUtil.copyProperties(userBillingInformation, userBillingInfoVO);
        return userBillingInfoVO;
    }

    @Override
    public void updateUserBillingInfo(Integer compdfKitId, UserBillingUpInfoDTO userBillingUpInfoDTO) {
        UserBillingInfoVO userBillingInfo = this.getUserBillingInfo(compdfKitId);
        UserBillingInformation userBillingInformation = new UserBillingInformation();
        BeanUtil.copyProperties(userBillingUpInfoDTO, userBillingInformation);
        if (BeanUtil.isEmpty(userBillingInfo)) {
            // 新增
            userBillingInformation.setCreatedAt(new Date());
            userBillingInformation.setUpdatedAt(new Date());
            userBillingInformation.setUserId(compdfKitId);
            this.baseMapper.insert(userBillingInformation);
        } else {
            // 修改
            this.baseMapper.update(null, Wrappers.<UserBillingInformation>lambdaUpdate()
                    .set(UserBillingInformation::getFirstName, userBillingUpInfoDTO.getFirstName())
                    .set(UserBillingInformation::getLastName, userBillingUpInfoDTO.getLastName())
                    .set(UserBillingInformation::getCompany, userBillingUpInfoDTO.getCompany())
                    .set(UserBillingInformation::getCountry, userBillingUpInfoDTO.getCountry())
                    .set(UserBillingInformation::getProvince, userBillingUpInfoDTO.getProvince())
                    .set(UserBillingInformation::getZip, userBillingUpInfoDTO.getZip())
                    .set(UserBillingInformation::getAddress,userBillingUpInfoDTO.getAddress())
                    .set(UserBillingInformation::getUpdatedAt, new Date())
                    .eq(UserBillingInformation::getUserId, compdfKitId));
//            this.baseMapper.update(userBillingInformation, new LambdaQueryWrapper<UserBillingInformation>().eq(UserBillingInformation::getUserId, compdfKitId));
        }
    }

    @Override
    public void updateUserBillingEmail(Integer compdfKitId, String email) {
        UserBillingInformation userBillingInformation = new UserBillingInformation();
        userBillingInformation.setEmail(email);
        if (BeanUtil.isNotEmpty(this.getUserBillingInfo(compdfKitId))) {
            userBillingInformation.setUpdatedAt(new Date());
            this.baseMapper.update(userBillingInformation, new LambdaQueryWrapper<UserBillingInformation>().eq(UserBillingInformation::getUserId, compdfKitId));
        } else {
            userBillingInformation.setUpdatedAt(new Date());
            userBillingInformation.setCreatedAt(new Date());
            userBillingInformation.setUserId(compdfKitId);
            this.baseMapper.insert(userBillingInformation);
        }
    }


    /**
     * 申请开发票
     *
     * @param compdfkitId     compdfkitId
     * @param applyInvoiceDTO applyInvoiceDTO
     * @return ApplyInvoiceVO
     */
    @Override
    public ApplyInvoiceVO applyInvoice(Integer compdfkitId, ApplyInvoiceDTO applyInvoiceDTO) {

        UserBillingInfoVO userBillingInfo = this.getUserBillingInfo(compdfkitId);
        // 发送邮箱是否填写
        if (StringUtils.isEmpty(userBillingInfo.getEmail())) {
            throw new CommonException(ResponseEnum.ERROR_BILL_EMAIL_NULL);
        }
        // 验证信息是否完整
        for (Field field : userBillingInfo.getClass().getDeclaredFields()) {
            // 设置允许访问私有属性
            field.setAccessible(true);
            // 检查属性值是否为null或空字符
            checkFieldNotNullOrEmpty(userBillingInfo, field);
        }
        // 生成发票号
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmssSSS");
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("MM/dd/yyyy");

        Date now = new Date();
        String billNo = sdf.format(now.getTime()).concat(String.valueOf(new Random().nextInt(9)));

        OrderBillingInfoVO orderBillingInfoVO = orderCashFlowService.getOrderBillingInfo(applyInvoiceDTO.getOrderId());
        // 是订阅
        String startDateStr = "";
        String endDateStr = "";
        String paymentCycle = "";
        String startDateAndEndDate = "";
        if (orderBillingInfoVO.getPackageType().equals(1)) {
            if ("Monthly".equals(orderBillingInfoVO.getSubscriptionType())) {
                LocalDateTime startDate = orderBillingInfoVO.getPayTime().plusMonths(orderBillingInfoVO.getPeriod()-1);
                LocalDateTime endDate = orderBillingInfoVO.getPayTime().plusMonths(orderBillingInfoVO.getPeriod());
                startDateStr = startDate.format(formatter);
                endDateStr = endDate.format(formatter);
                paymentCycle = "Monthly Payments";
            } else if ("Annually".equals(orderBillingInfoVO.getSubscriptionType())) {
                LocalDateTime startDate = orderBillingInfoVO.getPayTime().plusYears(orderBillingInfoVO.getPeriod()-1);
                LocalDateTime endDate = orderBillingInfoVO.getPayTime().plusYears(orderBillingInfoVO.getPeriod());
                startDateStr = startDate.format(formatter);
                endDateStr = endDate.format(formatter);
                paymentCycle = "Annually Payments";
            }
            startDateAndEndDate = startDateStr + " - " + endDateStr;
        } else if (orderBillingInfoVO.getPackageType().equals(2)) {
            paymentCycle = "Package Payments";
        }

        BillingInfoVO billingInfoVO = new BillingInfoVO(userBillingInfo);
        billingInfoVO.setStartDateAndEndDate(startDateAndEndDate);
        billingInfoVO.setPaymentCycle(paymentCycle);
        billingInfoVO.setGearLevel(orderBillingInfoVO.getGearLevel());
        BigDecimal totalTax = orderBillingInfoVO.getPrice().multiply(new BigDecimal("0.05")).setScale(2, BigDecimal.ROUND_HALF_UP);
        billingInfoVO.setPrice(orderBillingInfoVO.getPrice().subtract(totalTax).setScale(2, BigDecimal.ROUND_HALF_UP).toString());
        billingInfoVO.setTotalExcludingTax(totalTax.toString());
        billingInfoVO.setTotal(orderBillingInfoVO.getPrice().toString());
        billingInfoVO.setBillNo(billNo);
        // 01/15/2024

        LocalDate date = LocalDate.now();
        String invoiceDate = date.format(formatter);
        billingInfoVO.setInvoiceDate(invoiceDate);

        // 获取html模板
        // 填充数据
        String htmlStr = TemplatesUtil.replaceStringUsingFreeMarker(invoiceHtml, billingInfoVO);
        // 调用PHP接口生成PDF发票
        // 设置请求header
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        // 封装表单数据
        MultiValueMap<String, String> paramMap = new LinkedMultiValueMap<>();
        paramMap.add("html", htmlStr);
        String emailFileName;
        if (applyInvoiceDTO.getIsCN()) {
            emailFileName = "ComPDFKit_API_发票_"+billingInfoVO.getBillNo();
        }else {
            emailFileName = "ComPDFKit_API_Invoice_"+billingInfoVO.getBillNo();;
        }
        paramMap.add("fileName", emailFileName);
        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(paramMap, headers);
        // 发送POST请求
        Map postForObjectMap = restTemplate.postForObject(invoiceUrl, request, Map.class);
        String billUrl = postForObjectMap.get("url").toString();
        // TODO 上传资源服务器

        // 持久化发票数据
        orderCashFlowService.updateBillInfo(applyInvoiceDTO.getOrderId(), billUrl, billNo, JSONObject.toJSONString(userBillingInfo));
        // 返回结果
        ApplyInvoiceVO applyInvoiceVO = new ApplyInvoiceVO();
        applyInvoiceVO.setBillNo(billNo);
        applyInvoiceVO.setBillUrl(billUrl);
        return applyInvoiceVO;
    }

    @Override
    public void billSendEmail(BillSendEmailDTO billSendEmailDTO) {
        OrderBillVo orderBillVo = orderCashFlowService.getOrderCashFlowByOrderId(billSendEmailDTO.getOrderId());
        UserBillingInfoVO userBillingInfo = this.getUserBillingInfo(TokenUtil.getRequestHeader().getUsersId());
        try {
            // 发送邮件并发送附件
            MimeMessage message = javaMailSender.createMimeMessage();
            System.getProperties().setProperty("mail.mime.splitlongparameters", "false");
            //第2个参数:是否允许添加多部件
            MimeMessageHelper helper = new MimeMessageHelper(message, true);
            helper.setFrom(fromEmail);
            helper.setTo(billSendEmailDTO.getEmail());

            String emailContent;
            String emailFileName;
            if (billSendEmailDTO.getIsCN()) {
                emailFileName = "ComPDFKit_API_发票_"+orderBillVo.getBillNo();
                helper.setSubject("ComPDFKit API 发票号："+orderBillVo.getBillNo());
                emailContent = emailTemplate_CN.replaceAll(EMAIL_USER_NAME,userBillingInfo.getFirstName()
                        + " " + userBillingInfo.getLastName());
            }else {
                LocalDate date = LocalDate.now();
                // 01/15/2024
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("MM/dd/yyyy");
                String formattedDate = date.format(formatter);
                emailFileName = "ComPDFKit_API_Invoice_"+orderBillVo.getBillNo();
                helper.setSubject("Invoice "+orderBillVo.getBillNo()+" for ComPDFKit API due  "+formattedDate);
                //第2个参数:是否解析html
                emailContent = emailTemplate_EN.replaceAll(EMAIL_USER_NAME,userBillingInfo.getFirstName()
                        + " " + userBillingInfo.getLastName());
            }

            helper.setText(emailContent, true);
            //添加附件
            File file = new File(emailFileName + ".pdf");
            FileUtils.copyURLToFile(new URL(orderBillVo.getBillUrl()), file);
            // 对文件名进行MIME编码
            String encodedFileName = MimeUtility.encodeText(emailFileName + ".pdf");
            helper.addAttachment(encodedFileName, file);
            javaMailSender.send(message);
        } catch (MessagingException | IOException e) {
            log.error(e.getMessage(), e);
            throw new BusinessException("发票发送失败");
        }
    }

    @Override
    public List<OrderBillVo> getOrderBillVos(Integer compdfKitId) {
        // 获取用户所有成功的orderId
        List<Integer> successOrderIds = ordersService.getSuccessOrderIdByCompdfKitId(compdfKitId);
        List<OrderBillVo> orderCashFlowByOrderId = orderCashFlowService.getOrderCashFlowByOrderId(successOrderIds, MyConstant.ORDER_SUCCESS);
        // 获取用户所有退款的orderId
        List<Integer> failOrderIds = ordersService.getFailOrderIdByCompdfKitId(compdfKitId);
        orderCashFlowByOrderId.addAll(orderCashFlowService.getOrderCashFlowByOrderId(failOrderIds, MyConstant.ORDER_FAIL));
        // 按时间排序
        return orderCashFlowByOrderId.stream()
                .sorted(Comparator.comparing(OrderBillVo::getPayTime).reversed())
                .collect(Collectors.toList());
    }


    // 工具方法：判断属性是否为null或空字符
    private void checkFieldNotNullOrEmpty(Object obj, Field field) {
        try {
            Object value = field.get(obj);
            if (value == null || value.toString().isEmpty()) {
                throw new CommonException(ResponseEnum.ERROR_INCOMPLETE_BILLING_INFORMATION);
            }
        } catch (IllegalAccessException e) {
            log.error(e.getMessage(), e);
            throw new CommonException(ResponseEnum.ERROR_INCOMPLETE_BILLING_INFORMATION);
        }
    }

}
